﻿using System;
using System.Resources;
using System.Threading;
using System.Reflection;
using System.IO;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using RootPro.RootProCAD.Geometry;

/////////////////////////////////////////////////////////////////////////////////
// 図形のラバーバンドを描画するアドインサンプル
// コマンド実行中に図形のラバーバンドを描画します
// Copyright @ 2017 RootPro Co.,Ltd.
/////////////////////////////////////////////////////////////////////////////////
namespace RCAddInShapeRubberBandCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        string CircleRubberBandCommandName = "RootPro.AddIn.CircleRubberBandCS";     //コマンド名 

        private void AppAddIn_Startup(object sender, EventArgs e)
        {
			AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

			string[] parentMenus = { Properties.Resources.MenuName };

            CommandManager.AddMacroCommand("RCAddInShapeRubberBandCS", parentMenus, Properties.Resources.CommandName, MacroCommand);

            //図形コマンドを追加する
            Command command = CommandManager.AddCommand(CircleRubberBandCommandName, Properties.Resources.CommandName);
            AddParameters(command);

            //開始イベントと終了イベントの追加
            command.Begin += new CommandBeginEventHandler(command_Begin);
            command.End += new EventHandler(command_End);
        }

        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            //コマンドの削除
            CommandManager.RemoveMacroCommand(MacroCommand);
            CommandManager.RemoveCommand(CircleRubberBandCommandName);
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        /// <summary>
        /// パラメータを追加する
        /// </summary>
        /// <param name="command"></param>
        /// <param name="InputParameterType"></param>
        private void AddParameters(Command command)
        {
            //パラメータを追加する
            command.ParameterGroups.Add(ParameterItemType.Point, 0, Properties.Resources.CenterPointParameterName);
            PointParameterItem pointParameterItem = (PointParameterItem)command.GetParameterItemByID(0);
            pointParameterItem.Description = Properties.Resources.CenterPointParameterDescription;

            command.ParameterGroups.Add(ParameterItemType.Multiple, 1, Properties.Resources.RadiusParameterName);
            MultipleParameterItem multipleParameterItem = (MultipleParameterItem)command.GetParameterItemByID(1);
            multipleParameterItem.Description = Properties.Resources.RadiusParameterDescription;
            multipleParameterItem.ParameterTypeFlags = ParameterItemTypeFlags.Point | ParameterItemTypeFlags.Length;
        }

        /// <summary>
        /// コマンドの開始時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_Begin(Object sender, CommandBeginEventArgs e)
        {
            Command command = (Command)sender;

            //パラメータ変更イベントの追加
            command.ParameterChanged += new CommandParameterChangedEventHandler(command_ParameterChanged);
            //マウス移動イベントの追加
            command.MouseMove += new CommandMouseMoveEventHandler(command_MouseMove);
        }

        /// <summary>
        /// コマンドの終了時に呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void command_End(Object sender, EventArgs e)
        {
            Command command = (Command)sender;

            //パラメータ変更イベントの削除
            command.ParameterChanged -= new CommandParameterChangedEventHandler(command_ParameterChanged);
            //マウス移動イベントの削除
            command.MouseMove -= new CommandMouseMoveEventHandler(command_MouseMove);
        }

        /// <summary>
        /// マクロコマンドの実行
        /// </summary>
        private void MacroCommand()
        {
            //コマンドの実行
            CommandManager.ExecuteCommand(CircleRubberBandCommandName);
        }

        /// <summary>
        /// パラメータが変化したときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void command_ParameterChanged(Object sender, CommandParameterChangedEventArgs e)
        {
            Document doc = ActiveDocument;
            Command command = (Command)sender;

            // コマンドパラメータのチェック
            PointParameterItem pointParameterItem = (PointParameterItem)command.GetParameterItemByID(0);
            if (pointParameterItem.IsEmpty)
                return;

            MultipleParameterItem multipleParameterItem = (MultipleParameterItem)command.GetParameterItemByID(1);
            if (multipleParameterItem.IsEmpty)
                return;

            // UNDO 可能な操作を開始
            doc.UndoManager.BeginUndoUnit();

            Drawing drawing = doc.CurrentDrawing;

            // コマンドパラメータから中心点、半径を取得する
            PointParameterItem radiusPointItem = (PointParameterItem)multipleParameterItem.GetParameterItem(ParameterItemType.Point);
            LengthParameterItem lengthParameterItem = (LengthParameterItem)multipleParameterItem.GetParameterItem(ParameterItemType.Length);

            Double radius = 0;

            if (!lengthParameterItem.IsEmpty)
                radius = lengthParameterItem.Length;
            else if (!radiusPointItem.IsEmpty)
                radius = Geometry.GetDistance(pointParameterItem.Point, radiusPointItem.Point);

            // 円図形を追加
            drawing.Shapes.AddCircle(pointParameterItem.Point, radius);

            // UNDO 可能な操作を終了
            doc.UndoManager.EndUndoUnit();

            e.CommandState = CommandState.Execute;
        }

        /// <summary>
        /// マウスが移動したときに呼ばれる
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void command_MouseMove(Object sender, CommandMouseMoveEventArgs e)
        {
            Command command = (Command)sender;
            // Command.GetTemporaryParameterItemByID を使用してコマンドの一時的なパラメータアイテムを取得します。
            // 一時的なパラメータアイテムからは、すでにパラメータの入力が確定している場合は確定している入力値が、
            // 入力が確定していない場合はマウスの現在位置などの一時的な値が取得できます。

            // 中心点の一時パラメータを取得する
            PointParameterItem pointParameterItem = (PointParameterItem)command.GetTemporaryParameterItemByID(0);
            if (pointParameterItem.IsEmpty)
                return;

            // 半径/通過点の一時パラメータを取得する
            MultipleParameterItem multipleParameterItem = (MultipleParameterItem)command.GetTemporaryParameterItemByID(1);
            if (multipleParameterItem.IsEmpty)
                return;

            // 複合パラメータから半径を取得する
            double radius = 0;
            PointParameterItem radiusPointItem = (PointParameterItem)multipleParameterItem.GetParameterItem(ParameterItemType.Point);
            LengthParameterItem lengthParameterItem = (LengthParameterItem)multipleParameterItem.GetParameterItem(ParameterItemType.Length);

            if (!lengthParameterItem.IsEmpty)
                radius = lengthParameterItem.Length;
            else if (!radiusPointItem.IsEmpty)
                radius = Geometry.GetDistance(pointParameterItem.Point, radiusPointItem.Point);

            if (radius == 0)
                return;

            // ラバーバンド描画する円を作成
            CircleShape circle = ShapeFactory.CreateCircle(pointParameterItem.Point, radius);

            // ラバーバンドの色を設定
            circle.ColorNumber = 3;

            try
            {
                command.DrawShapeRubberBand(circle);
            }
            catch
            {
                throw;
            }
        }

		private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
		{
			AssemblyName assemblyName = new AssemblyName(args.Name);

			foreach (Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
			{
				if (AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
					return assembly;
			}

			if (assemblyName.Name.EndsWith(".resources"))
			{
				if (assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
				{
					string dllPath = Path.GetDirectoryName(this.AddInFullPath);
					dllPath = Path.Combine(dllPath, assemblyName.CultureName);
					dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

					if (File.Exists(dllPath))
					{
						try
						{
							return Assembly.LoadFrom(dllPath);
						}
						catch
						{
						}
					}
				}
			}

			return null;
		}
	}
}
